/* config.h
 *
 * Currently this must be edited by hand.
 * In the future, may be automatically generated.
 */

/* name of package */
#define PACKAGE        "edtile"
#define PACKAGE_TITLE  "EdTile"

/* version number of package */
#define VERSION      "0.2.5"
#define VERSION_HEX  0x025

/* platform */
#define LINUX 1
#undef WIN32

/* endianness */
#undef CPU_BIG_ENDIAN

/* disable debugging code ? */
#undef NDEBUG

/* whether we're using the FLTK toolkit */
#define USE_FLTK 1

/* whether we're using SDL */
#undef USE_SDL

/* whether we're using GL */
#undef USE_GL

/* whether we're using the PNG library */
#undef USE_PNG

/* whether we're using the JPEG library */
#undef USE_JPEG

/* do we have 64-bit integers ? */
#define HAVE_INT64

