/* A lexical scanner for NetHack generated by flex */

/* Scanner skeleton version:
 * flexhack.skl 3.3.0 (from .../flex/RCS/flex.skl,v 2.85 95/04/24 10:48:47)
 */
#define FLEXHACK_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5

#include "config.h"
#define yyconst const	/* some code inserted by flex will refer to yyconst */

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yy_start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yy_start - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart( yyin )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#define YY_BUF_SIZE 16384

typedef struct yy_buffer_state *YY_BUFFER_STATE;

extern int yyleng;
extern FILE *yyin, *yyout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

/* Return all but the first 'n' matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		*yy_cp = yy_hold_char; \
		yy_c_buf_p = yy_cp = yy_bp + n - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, yytext_ptr )

/* The following is because we cannot portably get our hands on size_t
 * (without autoconf's help, which isn't available because we want
 * flex-generated scanners to compile on their own).
 */
typedef unsigned int yy_size_t;


struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;
#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2
	};

static YY_BUFFER_STATE yy_current_buffer = 0;

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 */
#define YY_CURRENT_BUFFER yy_current_buffer

/* yy_hold_char holds the character lost when yytext is formed. */
static char yy_hold_char;

static int yy_n_chars;		/* number of characters read into yy_ch_buf */

int yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 1;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow yywrap()'s to do buffer switches
 * instead of setting up a fresh yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void FDECL(yyrestart, (FILE *));

void FDECL(yy_switch_to_buffer, (YY_BUFFER_STATE));
void NDECL(yy_load_buffer_state);
YY_BUFFER_STATE FDECL(yy_create_buffer, (FILE *,int));
void FDECL(yy_delete_buffer, (YY_BUFFER_STATE));
void FDECL(yy_init_buffer, (YY_BUFFER_STATE,FILE *));
void FDECL(yy_flush_buffer, (YY_BUFFER_STATE));
#define YY_FLUSH_BUFFER yy_flush_buffer( yy_current_buffer )

static genericptr_t FDECL(yy_flex_alloc, (yy_size_t));
static genericptr_t FDECL(yy_flex_realloc2, (genericptr_t,yy_size_t,int));
static void FDECL(yy_flex_free, (genericptr_t));

#define yy_new_buffer yy_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! yy_current_buffer ) \
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE ); \
	yy_current_buffer->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! yy_current_buffer ) \
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE ); \
	yy_current_buffer->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (yy_current_buffer->yy_at_bol)

typedef unsigned char YY_CHAR;
FILE *yyin = (FILE *) 0, *yyout = (FILE *) 0;
typedef int yy_state_type;
extern char *yytext;
#define yytext_ptr yytext

static yy_state_type NDECL(yy_get_previous_state);
static yy_state_type FDECL(yy_try_NUL_trans, (yy_state_type));
static int NDECL(yy_get_next_buffer);
static void FDECL(yy_fatal_error, (const char *));

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	yytext_ptr = yy_bp; \
	yyleng = (int) (yy_cp - yy_bp); \
	yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
	yy_c_buf_p = yy_cp;

#define YY_NUM_RULES 112
#define YY_END_OF_BUFFER 113
static yyconst short int yy_accept[640] =
    {   0,
        0,    0,    0,    0,  113,  111,  108,  107,  111,  111,
      111,  111,  105,    4,  111,  111,  111,  111,  111,  111,
      111,  111,  111,  111,  111,  111,  111,  111,  111,  111,
      111,  111,  111,  111,  111,  111,  111,  111,  111,  111,
      111,  111,  111,  111,  111,  111,  111,  111,  111,  111,
      111,    2,  111,  108,  111,  111,  105,  111,  111,  111,
      111,  111,  111,  111,  111,  111,  111,  111,  111,  108,
      107,    0,  106,    0,    0,  105,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
       87,    0,    0,    3,    0,    2,    2,    0,  108,    0,
      105,    0,    0,    0,    0,    0,    0,    0,    0,    2,
        0,    0,  110,    0,  110,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,   72,    0,    0,   67,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
       65,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,  109,    0,    0,    0,    0,    0,   17,
        0,    0,    0,    0,    0,   40,    0,    0,    0,    6,
        0,    0,   42,    0,    0,    0,   33,    0,    0,    0,

       36,   32,    0,    0,    0,   16,    0,    0,  104,    0,
        0,    0,    0,    0,    0,    0,    0,   93,    0,    0,
        0,    0,    0,    0,   88,   91,   51,    0,    0,    0,
        0,    0,    0,   60,    0,    0,    0,    0,    0,   94,
        0,    0,    0,    0,    0,    0,   55,    0,    0,    0,
       45,    0,    0,    0,    0,    0,    0,    0,    0,   90,
        0,    0,    0,   53,   12,    0,    0,    0,    0,    0,
       25,    0,    0,    0,    0,    0,    0,   10,    0,    0,
        0,    0,    8,    0,    0,    0,    7,    0,    0,    0,
        0,    0,    0,   27,    0,    0,    0,   59,   86,    0,

        0,   80,    0,    0,    0,    0,   74,    0,    0,    0,
        0,    0,   89,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,   50,    0,
        0,    0,   58,    0,   64,    0,    0,    0,   52,    0,
        0,   68,    0,    0,   30,   43,    0,    0,    0,    0,
        0,    0,    0,   26,    0,    0,    0,    0,    0,   13,
       28,    0,   21,    0,    0,    0,    0,   79,    0,   66,
       49,   62,   46,    0,    0,   97,    0,   69,    0,    0,
        0,    0,    0,   47,    0,    0,    0,    0,    0,    0,
       48,  101,    0,    0,   56,    0,   54,    0,    0,   85,

        0,    0,    1,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    5,   15,    0,    0,    0,   37,    0,   20,
        0,   95,    0,    0,   92,    0,    0,    0,   78,    0,
        0,    0,    0,   57,   73,   71,    0,    0,    0,   84,
        0,    0,    0,    0,   39,    0,    0,   31,   11,    9,
       19,    0,    0,    0,    0,    0,    0,    0,  102,    0,
        0,    0,    0,    0,    0,    0,    0,   83,    0,    0,
       77,    0,   96,   70,   14,    0,   41,    0,    0,    0,
        0,    0,    0,    0,   75,   98,   61,    0,  100,   44,
       81,   82,    0,    0,    0,   18,    0,    0,    0,    0,

        0,    0,    0,   63,    0,   99,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,   34,   35,    0,    0,
        0,    0,    0,   76,  103,    0,    0,    0,   24,    0,
        0,    0,   22,    0,    0,   23,   29,   38,    0
    } ;

static yyconst int yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    4,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    5,    1,    6,    7,    1,    8,    1,    9,    1,
        1,    1,   10,    1,   11,   12,    1,   13,   13,   13,
       13,   13,   13,   13,   13,   13,   13,   14,    1,    1,
        1,    1,    1,    1,   15,   16,   17,   18,   19,   20,
       21,   22,   23,   24,   25,   26,   27,   28,   29,   30,
        1,   31,   32,   33,   34,   35,   36,    1,   37,   38,
       39,   40,   41,    1,   42,    1,   43,   44,   45,   46,

       47,   48,   49,   50,   51,   52,   53,   54,   55,   56,
       57,   58,    1,   59,   60,   61,   62,   63,   64,    1,
        1,    1,   12,   12,   12,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst int yy_meta[65] =
    {   0,
        1,    2,    3,    2,    2,    1,    2,    1,    1,    2,
        2,    2,    2,    1,    2,    2,    2,    1,    1,    2,
        1,    2,    2,    1,    2,    2,    1,    1,    1,    2,
        1,    2,    2,    1,    1,    2,    1,    1,    1,    2,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1
    } ;

static yyconst short int yy_base[645] =
    {   0,
        0,   58,   83,   62,  796,  797,   65,  797,  792,  788,
      753,  779,  778,  797,  764,  758,   44,   43,  760,   42,
       62,  759,   60,   63,   68,  770,  756,   92,   91,   91,
      769,   71,   72,   76,   87,   55,   84,   77,   61,   96,
      103,   95,  104,  103,  108,  111,   99,  107,  736,  779,
      151,  797,  778,  169,  173,  179,  182,  185,  194,  197,
      752,  180,  185,  193,  181,  194,  202,  214,  241,   75,
      797,  773,  797,  769,  768,  763,  742,  759,  758,  136,
      743,  756,  749,  754,  734,  738,  740,  742,  746,  728,
      724,  729,  732,  732,  151,  734,  162,  729,  735,  726,

      726,  738,  736,  725,  735,  723,  225,  224,  143,  704,
      693,  703,  698,  683,  686,  683,  685,  697,  682,  162,
      679,  673,  676,  675,  685,  679,  678,  180,  671,  666,
      172,  668,  683,  192,  668,  670,  663,  229,  672,  676,
      679,  678,  664,  670,  662,  203,  655,  658,  653,  235,
      797,  654,  710,  797,  212,  797,  797,  709,  274,  264,
      265,  269,  277,  247,  282,  283,  285,  293,  294,  797,
      708,    0,  797,  701,  700,  693,  679,  678,  672,  673,
      672,  666,  670,  679,  671,  671,  679,  663,  677,  675,
      674,  660,  659,  671,  674,  646,  668,  660,  652,  666,

      660,  655,  656,  657,  648,  659,  647,  650,  254,  626,
      631,  616,  625,  618,  610,  608,  615,  611,  605,  608,
      604,  609,  601,  601,  604,  598,  597,  598,  596,  601,
      606,  607,  591,  797,  590,  591,  797,  596,  601,  590,
      602,  592,  584,  582,  588,  584,  585,  272,  578,  591,
      590,  580,  590,  589,  587,  582,  586,  571,  578,  567,
      797,  580,  564,  574,  573,  562,  266,  303,  299,  595,
      308,  311,  309,  797,  590,  603,  602,  603,  594,  797,
      600,  600,  582,  580,  593,  797,  569,  591,  583,  572,
      592,  573,  797,  575,  306,  587,  797,  588,  573,  572,

      797,  797,  569,  570,  568,  797,  574,  304,  797,  540,
      536,  535,  546,  545,  531,  533,  542,  797,  541,  527,
      539,  534,  541,  536,  797,  797,  797,  539,  534,  533,
      568,  530,  526,  797,  529,  528,  531,  517,  520,  797,
      510,  511,  518,  511,  524,  509,  797,  515,  510,  518,
      797,  515,  514,  503,  498,  497,  496,  500,  505,  797,
      495,  499,  491,  797,  797,  548,  317,  535,  320,  321,
      797,  527,  529,  524,  528,  514,  509,  797,  528,  509,
      514,  509,  797,  524,  517,  518,  797,  513,  520,  501,
      507,  505,  503,  797,  501,  500,  508,  797,  797,  480,

      468,  797,  478,  469,  467,  463,  797,  475,  471,  468,
      472,  454,  797,  470,  293,  461,  460,  464,  466,  450,
      450,  462,  461,  464,  457,  446,  446,  460,  797,  455,
      440,  452,  797,  444,  797,  436,  437,  449,  797,  435,
      440,  797,  463,  333,  797,  797,  464,  462,  467,  466,
      465,  456,  471,  797,  459,  465,  452,  461,  449,  797,
      797,  438,  797,  452,  447,  440,  433,  797,  429,  797,
      797,  797,  797,  418,  417,  797,  425,  797,  424,  419,
      412,  421,  416,  797,  404,  404,  419,  404,  408,  405,
      797,  797,  406,  401,  797,  396,  797,  402,  405,  797,

      408,  407,  797,  332,  434,  421,  433,  422,  421,  411,
      417,  421,  797,  797,  424,  412,  341,  797,  410,  797,
      388,  797,  394,  393,  797,  391,  389,  380,  797,  379,
      376,  387,  372,  797,  797,  797,  381,  374,  376,  797,
      380,  382,  381,  395,  797,  404,  403,  797,  797,  797,
      797,  408,  386,  392,  391,  403,  392,  375,  797,  370,
      369,  353,  363,  353,  355,  363,  350,  797,  359,  348,
      797,  356,  797,  797,  797,  386,  797,  388,  388,  371,
      373,  376,  384,  367,  797,  797,  797,  336,  797,  797,
      797,  797,  340,  334,  333,  797,  367,  366,  360,  358,

      370,  371,  368,  797,  339,  797,  338,  365,  341,  343,
      332,  347,  344,  339,  313,  311,  797,  797,  338,  322,
      293,  278,  277,  797,  797,  253,  224,  209,  797,  161,
      138,  123,  797,  101,   69,  797,  797,  797,  797,  371,
      374,  376,  378,  381
    } ;

static yyconst short int yy_def[645] =
    {   0,
      639,    1,    1,    3,  639,  639,  639,  639,  639,  640,
      641,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  642,
      639,  639,  639,  643,  643,  643,  643,  643,  643,  643,
      639,   60,   60,   60,   60,   60,   60,   60,  642,  639,
      639,  640,  639,  639,  644,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,

      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  642,  639,  639,  639,  639,  639,   60,   60,
       60,   60,   60,  639,   60,   60,   60,   60,   60,  639,
      642,   69,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,

      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,   60,   60,  639,
       60,   60,   60,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,

      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,   60,  639,   60,   60,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,

      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,   60,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,

      639,  639,  639,   60,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,

      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,    0,  639,
      639,  639,  639,  639
    } ;

static yyconst short int yy_nxt[862] =
    {   0,
        6,    7,    8,    9,    7,   10,    6,    6,   11,   12,
       12,    6,   13,   14,   15,   16,   17,   18,   19,   20,
       21,    6,   22,    6,    6,   23,   24,   25,   26,   27,
       28,   29,   30,    6,    6,   31,    6,    6,   32,    6,
        6,    6,   33,   34,   35,   36,   37,   38,    6,   39,
        6,    6,    6,   40,   41,   42,   43,   44,   45,   46,
       47,   48,    6,   49,   50,   79,   70,   84,   69,   70,
       85,   81,   80,   82,   89,  107,   70,   91,   90,   70,
       86,   92,   94,  108,   51,   52,   53,   54,   51,   55,
       87,   93,   56,   56,   55,   57,   95,   58,   59,   60,

      638,   61,   62,  128,   55,   63,   98,   55,   64,  104,
       99,  122,   65,  101,   66,   67,  123,  129,   68,  126,
      100,  105,   55,  102,  103,  109,  124,  127,  637,  113,
      110,  111,  114,  117,  115,  112,  118,  116,  130,  125,
      119,  137,  131,  120,  134,  135,  132,  139,  121,  141,
      143,  138,  133,  145,  636,  148,  142,  149,  144,  136,
      146,  140,  150,  179,  151,  155,  180,  147,  635,   92,
       70,  157,  158,  159,  639,  157,  158,  195,  196,   93,
      639,  157,  158,  639,  157,  158,  639,  157,  158,  634,
      198,  161,  199,  210,  161,  639,  157,  158,  639,  157,

      158,  160,  160,  211,  222,  165,  160,  166,   85,   97,
      162,   90,   88,  639,  160,  160,  167,  223,  163,  235,
      104,  639,  639,  160,   78,   80,  168,  103,  169,  107,
      639,  209,  105,  231,  236,  160,  108,  108,  232,  239,
      633,  267,  639,  170,  171,  172,  240,  172,  241,  191,
      172,  172,  172,  172,  632,  172,  172,  172,  308,  258,
      172,  259,  172,  172,  270,  172,  172,  183,  365,  366,
      172,  245,  172,  172,  246,   70,  172,  161,  159,  263,
      172,  631,  264,  247,  248,  160,  160,  249,  265,  250,
      160,  269,  639,  639,  309,  160,  271,  639,  160,  630,

      189,  268,  639,  160,  160,  639,  160,  272,  308,  629,
      639,  639,  203,  639,  160,  160,  346,  367,  347,  273,
      160,  639,  639,  388,  160,  628,  277,  639,  283,  160,
      160,  639,  160,  369,  370,  389,  639,  639,  160,  639,
      627,  160,  160,  444,  309,  639,  479,  371,  639,  639,
      394,  480,  504,  160,  160,  626,  625,  554,  624,  623,
      639,  639,  622,  621,  620,  619,  618,  555,  520,  556,
      557,   72,   72,   72,   74,   74,  153,  153,  153,  160,
      160,  174,  174,  617,  616,  615,  614,  613,  612,  611,
      610,  609,  608,  607,  606,  605,  604,  603,  602,  601,

      600,  599,  598,  597,  596,  595,  594,  593,  592,  591,
      590,  589,  588,  587,  586,  585,  584,  583,  582,  581,
      580,  579,  578,  577,  576,  575,  574,  573,  572,  571,
      570,  569,  568,  567,  566,  565,  564,  563,  562,  561,
      560,  559,  558,  553,  552,  551,  550,  549,  548,  547,
      546,  545,  544,  543,  542,  541,  540,  539,  538,  537,
      536,  535,  534,  533,  532,  531,  530,  529,  528,  527,
      526,  525,  524,  523,  522,  521,  520,  519,  518,  517,
      516,  515,  514,  513,  512,  511,  510,  509,  508,  507,
      506,  505,  503,  502,  501,  500,  499,  498,  497,  496,

      495,  494,  493,  492,  491,  490,  489,  488,  487,  486,
      485,  484,  483,  482,  481,  478,  477,  476,  475,  474,
      473,  472,  471,  470,  469,  468,  467,  466,  465,  464,
      463,  462,  461,  460,  459,  458,  457,  456,  455,  454,
      453,  452,  451,  450,  449,  448,  447,  446,  445,  443,
      365,  442,  441,  440,  439,  438,  437,  436,  435,  434,
      433,  432,  431,  430,  429,  428,  427,  426,  425,  424,
      423,  422,  421,  420,  419,  418,  417,  416,  415,  414,
      413,  412,  411,  410,  409,  408,  407,  406,  405,  404,
      403,  402,  401,  400,  399,  398,  397,  396,  395,  394,

      393,  392,  391,  390,  387,  386,  385,  384,  383,  382,
      381,  380,  379,  378,  377,  376,  375,  374,  373,  372,
      371,  368,  364,  363,  362,  361,  360,  359,  358,  357,
      356,  355,  354,  353,  352,  351,  350,  349,  348,  345,
      344,  343,  342,  341,  340,  339,  338,  337,  336,  335,
      334,  333,  332,  331,  330,  329,  328,  327,  326,  325,
      324,  323,  322,  321,  320,  319,  318,  317,  316,  315,
      314,  313,  312,  311,  310,  307,  306,  305,  304,  303,
      302,  301,  300,  299,  298,  297,  296,  295,  294,  293,
      292,  291,  290,  289,  288,  287,  286,  285,  284,  283,

      282,  281,  280,  279,  278,  277,  276,  275,  274,  274,
      170,  157,  154,  266,  262,  261,  260,  257,  256,  255,
      254,  253,  252,  251,  244,  243,  242,  238,  237,  234,
      233,  230,  229,  228,  227,  226,  225,  224,  221,  220,
      219,  218,  217,  216,  215,  214,  213,  212,  208,  207,
      206,  205,  204,  203,  202,  201,  200,  197,  194,  193,
      192,  191,  190,  189,  188,  187,  186,  185,  184,  183,
      182,  181,  178,  177,  176,   76,  175,  173,   73,  164,
      156,  154,  152,  106,   97,   96,   88,   83,   78,   77,
       76,   76,   75,   73,   71,  639,    5,  639,  639,  639,

      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639
    } ;

static yyconst short int yy_chk[862] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    2,   17,    7,   20,    4,    7,
       20,   18,   17,   18,   23,   32,   70,   24,   23,   70,
       21,   24,   25,   32,    2,    3,    3,    3,    4,    3,
       21,   24,    3,    3,    3,    3,   25,    3,    3,    3,

      635,    3,    3,   39,    3,    3,   28,    3,    3,   30,
       28,   36,    3,   29,    3,    3,   36,   39,    3,   38,
       28,   30,    3,   29,   29,   33,   37,   38,  634,   34,
       33,   33,   34,   35,   34,   33,   35,   34,   40,   37,
       35,   42,   40,   35,   41,   41,   40,   43,   35,   44,
       45,   42,   40,   46,  632,   47,   44,   47,   45,   41,
       46,   43,   48,   80,   48,   51,   80,   46,  631,   51,
       54,   54,   54,   54,   55,   55,   55,   95,   95,   51,
       56,   56,   56,   57,   57,   57,   58,   58,   58,  630,
       97,   56,   97,  109,   57,   59,   59,   59,   60,   60,

       60,   62,   65,  109,  120,   62,   63,   64,   62,   65,
       58,   64,   63,   63,   64,   66,   66,  120,   60,  131,
       67,   64,   66,   67,   59,   60,   66,   66,   68,  107,
       67,  108,   67,  128,  131,   68,  108,  107,  128,  134,
      628,  155,   68,   69,   69,   69,  134,   69,  134,  155,
       69,   69,   69,   69,  627,   69,   69,   69,  209,  146,
       69,  146,   69,   69,  164,   69,   69,  164,  267,  267,
       69,  138,   69,   69,  138,  159,   69,  161,  159,  150,
       69,  626,  150,  138,  138,  160,  161,  138,  150,  138,
      162,  163,  160,  161,  209,  159,  165,  162,  163,  623,

      166,  162,  159,  165,  166,  163,  167,  168,  308,  622,
      165,  166,  167,  167,  168,  169,  248,  268,  248,  169,
      269,  168,  169,  295,  268,  621,  269,  269,  271,  271,
      273,  268,  272,  272,  273,  295,  271,  273,  367,  272,
      620,  369,  370,  370,  308,  367,  415,  367,  369,  370,
      369,  415,  444,  504,  444,  619,  616,  517,  615,  614,
      504,  444,  613,  612,  611,  610,  609,  517,  504,  517,
      517,  640,  640,  640,  641,  641,  642,  642,  642,  643,
      643,  644,  644,  608,  607,  605,  603,  602,  601,  600,
      599,  598,  597,  595,  594,  593,  588,  584,  583,  582,

      581,  580,  579,  578,  576,  572,  570,  569,  567,  566,
      565,  564,  563,  562,  561,  560,  558,  557,  556,  555,
      554,  553,  552,  547,  546,  544,  543,  542,  541,  539,
      538,  537,  533,  532,  531,  530,  528,  527,  526,  524,
      523,  521,  519,  516,  515,  512,  511,  510,  509,  508,
      507,  506,  505,  502,  501,  499,  498,  496,  494,  493,
      490,  489,  488,  487,  486,  485,  483,  482,  481,  480,
      479,  477,  475,  474,  469,  467,  466,  465,  464,  462,
      459,  458,  457,  456,  455,  453,  452,  451,  450,  449,
      448,  447,  443,  441,  440,  438,  437,  436,  434,  432,

      431,  430,  428,  427,  426,  425,  424,  423,  422,  421,
      420,  419,  418,  417,  416,  414,  412,  411,  410,  409,
      408,  406,  405,  404,  403,  401,  400,  397,  396,  395,
      393,  392,  391,  390,  389,  388,  386,  385,  384,  382,
      381,  380,  379,  377,  376,  375,  374,  373,  372,  368,
      366,  363,  362,  361,  359,  358,  357,  356,  355,  354,
      353,  352,  350,  349,  348,  346,  345,  344,  343,  342,
      341,  339,  338,  337,  336,  335,  333,  332,  331,  330,
      329,  328,  324,  323,  322,  321,  320,  319,  317,  316,
      315,  314,  313,  312,  311,  310,  307,  305,  304,  303,

      300,  299,  298,  296,  294,  292,  291,  290,  289,  288,
      287,  285,  284,  283,  282,  281,  279,  278,  277,  276,
      275,  270,  266,  265,  264,  263,  262,  260,  259,  258,
      257,  256,  255,  254,  253,  252,  251,  250,  249,  247,
      246,  245,  244,  243,  242,  241,  240,  239,  238,  236,
      235,  233,  232,  231,  230,  229,  228,  227,  226,  225,
      224,  223,  222,  221,  220,  219,  218,  217,  216,  215,
      214,  213,  212,  211,  210,  208,  207,  206,  205,  204,
      203,  202,  201,  200,  199,  198,  197,  196,  195,  194,
      193,  192,  191,  190,  189,  188,  187,  186,  185,  184,

      183,  182,  181,  180,  179,  178,  177,  176,  175,  174,
      171,  158,  153,  152,  149,  148,  147,  145,  144,  143,
      142,  141,  140,  139,  137,  136,  135,  133,  132,  130,
      129,  127,  126,  125,  124,  123,  122,  121,  119,  118,
      117,  116,  115,  114,  113,  112,  111,  110,  106,  105,
      104,  103,  102,  101,  100,   99,   98,   96,   94,   93,
       92,   91,   90,   89,   88,   87,   86,   85,   84,   83,
       82,   81,   79,   78,   77,   76,   75,   74,   72,   61,
       53,   50,   49,   31,   27,   26,   22,   19,   16,   15,
       13,   12,   11,   10,    9,    5,  639,  639,  639,  639,

      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639,  639,  639,  639,  639,  639,  639,  639,  639,  639,
      639
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
char *yytext;
#define INITIAL 0
/*	SCCS Id: @(#)lev_lex.c	3.4	2002/03/27	*/
/*	Copyright (c) 1989 by Jean-Christophe Collet */
/* NetHack may be freely redistributed.  See license for details. */

#define LEV_LEX_C

#include "hack.h"
#include "lev_comp.h"
#include "sp_lev.h"

/* Most of these don't exist in flex, yywrap is macro and
 * yyunput is properly declared in flex.skel.
 */
#if !defined(FLEX_SCANNER) && !defined(FLEXHACK_SCANNER)
int FDECL(yyback, (int *,int));
int NDECL(yylook);
int NDECL(yyinput);
int NDECL(yywrap);
int NDECL(yylex);
	/* Traditional lexes let yyunput() and yyoutput() default to int;
	 * newer ones may declare them as void since they don't return
	 * values.  For even more fun, the lex supplied as part of the
	 * newer unbundled compiler for SunOS 4.x adds the void declarations
	 * (under __STDC__ or _cplusplus ifdefs -- otherwise they remain
	 * int) while the bundled lex and the one with the older unbundled
	 * compiler do not.  To detect this, we need help from outside --
	 * sys/unix/Makefile.utl.
	 *
	 * Digital UNIX is difficult and still has int in spite of all
	 * other signs.
	 */
# if defined(NeXT) || defined(SVR4) || defined(_AIX32)
#  define VOIDYYPUT
# endif
# if !defined(VOIDYYPUT) && defined(POSIX_TYPES)
#  if !defined(BOS) && !defined(HISX) && !defined(_M_UNIX) && !defined(VMS)
#   define VOIDYYPUT
#  endif
# endif
# if !defined(VOIDYYPUT) && defined(WEIRD_LEX)
#  if defined(SUNOS4) && defined(__STDC__) && (WEIRD_LEX > 1)
#   define VOIDYYPUT
#  endif
# endif
# if defined(VOIDYYPUT) && defined(__osf__)
#  undef VOIDYYPUT
# endif
# ifdef VOIDYYPUT
void FDECL(yyunput, (int));
void FDECL(yyoutput, (int));
# else
int FDECL(yyunput, (int));
int FDECL(yyoutput, (int));
# endif
#endif	/* !FLEX_SCANNER && !FLEXHACK_SCANNER */

#ifdef FLEX_SCANNER
#define YY_MALLOC_DECL \
	      genericptr_t FDECL(malloc, (size_t)); \
	      genericptr_t FDECL(realloc, (genericptr_t,size_t));
#endif

void FDECL(init_yyin, (FILE *));
void FDECL(init_yyout, (FILE *));

/*
 * This doesn't always get put in lev_comp.h
 * (esp. when using older versions of bison).
 */
extern YYSTYPE yylval;

int line_number = 1, colon_line_number = 1;
static char map[4096];
static int map_cnt = 0;

#define MAPC 1


/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
extern int NDECL(yywrap);
#endif

#ifndef YY_NO_UNPUT
static void FDECL(yyunput, (int,char *));
#endif

#ifndef yytext_ptr
static void FDECL(yy_flex_strncpy, (char *,const char *,int));
#endif

#ifndef YY_NO_INPUT
static int NDECL(input);
#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */

#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO (void) fwrite( yytext, yyleng, 1, yyout )
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( yy_current_buffer->yy_is_interactive ) \
		{ \
		int c = '*', n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else if ( ((result = fread( buf, 1, max_size, yyin )) == 0) \
		  && ferror( yyin ) ) \
		YY_FATAL_ERROR( "input in flex scanner failed" );
#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	if ( yyleng > 0 ) \
		yy_current_buffer->yy_at_bol = \
				(yytext[yyleng - 1] == '\n'); \
	YY_USER_ACTION

int NDECL(yylex);
int yylex()
	{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;



	if ( yy_init )
		{
		yy_init = 0;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! yy_start )
			yy_start = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( ! yy_current_buffer )
			yy_current_buffer =
				yy_create_buffer( yyin, YY_BUF_SIZE );

		yy_load_buffer_state();
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = yy_c_buf_p;

		/* Support of yytext. */
		*yy_cp = yy_hold_char;

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = yy_start;
		yy_current_state += YY_AT_BOL();
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				yy_last_accepting_state = yy_current_state;
				yy_last_accepting_cpos = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 640 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 797 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = yy_last_accepting_cpos;
			yy_current_state = yy_last_accepting_state;
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;


do_action:	/* This label is used only to access EOF actions. */


		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = yy_hold_char;
			yy_cp = yy_last_accepting_cpos;
			yy_current_state = yy_last_accepting_state;
			goto yy_find_action;

case 1:
YY_RULE_SETUP
{
		  BEGIN(INITIAL);
		  yylval.map = (char *) alloc(map_cnt + 1);
		  (void) strncpy(yylval.map, map, map_cnt);
		  yylval.map[map_cnt] = 0;
		  map_cnt = 0;
		  return MAP_ID;
		}
	YY_BREAK
case 2:
YY_RULE_SETUP
{
		  int len = yyleng;
		  /* convert \r\n to \n */
		  if (len >= 2 && yytext[len - 2] == '\r') len -= 1;
		  line_number++;
		  (void) strncpy(map + map_cnt, yytext, len);
		  map_cnt += len;
		  map[map_cnt - 1] = '\n';
		  map[map_cnt] = '\0';
		}
	YY_BREAK
case 3:
YY_RULE_SETUP
{ line_number++; }
	YY_BREAK
case 4:
YY_RULE_SETUP
{ colon_line_number = line_number; return ':'; }
	YY_BREAK
case 5:
YY_RULE_SETUP
return MESSAGE_ID;
	YY_BREAK
case 6:
YY_RULE_SETUP
return MAZE_ID;
	YY_BREAK
case 7:
YY_RULE_SETUP
return NOMAP_ID;
	YY_BREAK
case 8:
YY_RULE_SETUP
return LEVEL_ID;
	YY_BREAK
case 9:
YY_RULE_SETUP
return LEV_INIT_ID;
	YY_BREAK
case 10:
YY_RULE_SETUP
return FLAGS_ID;
	YY_BREAK
case 11:
YY_RULE_SETUP
return GEOMETRY_ID;
	YY_BREAK
case 12:
YY_RULE_SETUP
{ BEGIN(MAPC); line_number++; }
	YY_BREAK
case 13:
YY_RULE_SETUP
return OBJECT_ID;
	YY_BREAK
case 14:
YY_RULE_SETUP
return COBJECT_ID;
	YY_BREAK
case 15:
YY_RULE_SETUP
return MONSTER_ID;
	YY_BREAK
case 16:
YY_RULE_SETUP
return TRAP_ID;
	YY_BREAK
case 17:
YY_RULE_SETUP
return DOOR_ID;
	YY_BREAK
case 18:
YY_RULE_SETUP
return DRAWBRIDGE_ID;
	YY_BREAK
case 19:
YY_RULE_SETUP
return MAZEWALK_ID;
	YY_BREAK
case 20:
YY_RULE_SETUP
return WALLIFY_ID;
	YY_BREAK
case 21:
YY_RULE_SETUP
return REGION_ID;
	YY_BREAK
case 22:
YY_RULE_SETUP
return RANDOM_OBJECTS_ID;
	YY_BREAK
case 23:
YY_RULE_SETUP
return RANDOM_MONSTERS_ID;
	YY_BREAK
case 24:
YY_RULE_SETUP
return RANDOM_PLACES_ID;
	YY_BREAK
case 25:
YY_RULE_SETUP
return ALTAR_ID;
	YY_BREAK
case 26:
YY_RULE_SETUP
return LADDER_ID;
	YY_BREAK
case 27:
YY_RULE_SETUP
return STAIR_ID;
	YY_BREAK
case 28:
YY_RULE_SETUP
return PORTAL_ID;
	YY_BREAK
case 29:
YY_RULE_SETUP
return TELEPRT_ID;
	YY_BREAK
case 30:
YY_RULE_SETUP
return BRANCH_ID;
	YY_BREAK
case 31:
YY_RULE_SETUP
return FOUNTAIN_ID;
	YY_BREAK
case 32:
YY_RULE_SETUP
return SINK_ID;
	YY_BREAK
case 33:
YY_RULE_SETUP
return POOL_ID;
	YY_BREAK
case 34:
YY_RULE_SETUP
return NON_DIGGABLE_ID;
	YY_BREAK
case 35:
YY_RULE_SETUP
return NON_PASSWALL_ID;
	YY_BREAK
case 36:
YY_RULE_SETUP
return ROOM_ID;
	YY_BREAK
case 37:
YY_RULE_SETUP
return SUBROOM_ID;
	YY_BREAK
case 38:
YY_RULE_SETUP
return RAND_CORRIDOR_ID;
	YY_BREAK
case 39:
YY_RULE_SETUP
return CORRIDOR_ID;
	YY_BREAK
case 40:
YY_RULE_SETUP
return GOLD_ID;
	YY_BREAK
case 41:
YY_RULE_SETUP
return ENGRAVING_ID;
	YY_BREAK
case 42:
YY_RULE_SETUP
return NAME_ID;
	YY_BREAK
case 43:
YY_RULE_SETUP
return CHANCE_ID;
	YY_BREAK
case 44:
YY_RULE_SETUP
return LEV;
	YY_BREAK
case 45:
YY_RULE_SETUP
{ yylval.i=D_ISOPEN; return DOOR_STATE; }
	YY_BREAK
case 46:
YY_RULE_SETUP
{ yylval.i=D_CLOSED; return DOOR_STATE; }
	YY_BREAK
case 47:
YY_RULE_SETUP
{ yylval.i=D_LOCKED; return DOOR_STATE; }
	YY_BREAK
case 48:
YY_RULE_SETUP
{ yylval.i=D_NODOOR; return DOOR_STATE; }
	YY_BREAK
case 49:
YY_RULE_SETUP
{ yylval.i=D_BROKEN; return DOOR_STATE; }
	YY_BREAK
case 50:
YY_RULE_SETUP
{ yylval.i=W_NORTH; return DIRECTION; }
	YY_BREAK
case 51:
YY_RULE_SETUP
{ yylval.i=W_EAST; return DIRECTION; }
	YY_BREAK
case 52:
YY_RULE_SETUP
{ yylval.i=W_SOUTH; return DIRECTION; }
	YY_BREAK
case 53:
YY_RULE_SETUP
{ yylval.i=W_WEST; return DIRECTION; }
	YY_BREAK
case 54:
YY_RULE_SETUP
{ yylval.i = -1; return RANDOM_TYPE; }
	YY_BREAK
case 55:
YY_RULE_SETUP
{ yylval.i = -2; return NONE; }
	YY_BREAK
case 56:
YY_RULE_SETUP
return O_REGISTER;
	YY_BREAK
case 57:
YY_RULE_SETUP
return M_REGISTER;
	YY_BREAK
case 58:
YY_RULE_SETUP
return P_REGISTER;
	YY_BREAK
case 59:
YY_RULE_SETUP
return A_REGISTER;
	YY_BREAK
case 60:
YY_RULE_SETUP
{ yylval.i=1; return LEFT_OR_RIGHT; }
	YY_BREAK
case 61:
YY_RULE_SETUP
{ yylval.i=2; return LEFT_OR_RIGHT; }
	YY_BREAK
case 62:
YY_RULE_SETUP
{ yylval.i=3; return CENTER; }
	YY_BREAK
case 63:
YY_RULE_SETUP
{ yylval.i=4; return LEFT_OR_RIGHT; }
	YY_BREAK
case 64:
YY_RULE_SETUP
{ yylval.i=5; return LEFT_OR_RIGHT; }
	YY_BREAK
case 65:
YY_RULE_SETUP
{ yylval.i=1; return TOP_OR_BOT; }
	YY_BREAK
case 66:
YY_RULE_SETUP
{ yylval.i=5; return TOP_OR_BOT; }
	YY_BREAK
case 67:
YY_RULE_SETUP
{ yylval.i=1; return LIGHT_STATE; }
	YY_BREAK
case 68:
YY_RULE_SETUP
{ yylval.i=0; return LIGHT_STATE; }
	YY_BREAK
case 69:
YY_RULE_SETUP
{ yylval.i=0; return FILLING; }
	YY_BREAK
case 70:
YY_RULE_SETUP
{ yylval.i=1; return FILLING; }
	YY_BREAK
case 71:
YY_RULE_SETUP
{ yylval.i= AM_NONE; return ALIGNMENT; }
	YY_BREAK
case 72:
YY_RULE_SETUP
{ yylval.i= AM_LAWFUL; return ALIGNMENT; }
	YY_BREAK
case 73:
YY_RULE_SETUP
{ yylval.i= AM_NEUTRAL; return ALIGNMENT; }
	YY_BREAK
case 74:
YY_RULE_SETUP
{ yylval.i= AM_CHAOTIC; return ALIGNMENT; }
	YY_BREAK
case 75:
YY_RULE_SETUP
{ yylval.i= AM_SPLEV_CO; return ALIGNMENT; }
	YY_BREAK
case 76:
YY_RULE_SETUP
{ yylval.i= AM_SPLEV_NONCO; return ALIGNMENT; }
	YY_BREAK
case 77:
YY_RULE_SETUP
{ yylval.i=1; return MON_ATTITUDE; }
	YY_BREAK
case 78:
YY_RULE_SETUP
{ yylval.i=0; return MON_ATTITUDE; }
	YY_BREAK
case 79:
YY_RULE_SETUP
{ yylval.i=1; return MON_ALERTNESS; }
	YY_BREAK
case 80:
YY_RULE_SETUP
{ yylval.i=0; return MON_ALERTNESS; }
	YY_BREAK
case 81:
YY_RULE_SETUP
{ yylval.i= M_AP_FURNITURE; return MON_APPEARANCE; }
	YY_BREAK
case 82:
YY_RULE_SETUP
{ yylval.i= M_AP_MONSTER;   return MON_APPEARANCE; }
	YY_BREAK
case 83:
YY_RULE_SETUP
{ yylval.i= M_AP_OBJECT;    return MON_APPEARANCE; }
	YY_BREAK
case 84:
YY_RULE_SETUP
{ yylval.i=2; return ALTAR_TYPE; }
	YY_BREAK
case 85:
YY_RULE_SETUP
{ yylval.i=1; return ALTAR_TYPE; }
	YY_BREAK
case 86:
YY_RULE_SETUP
{ yylval.i=0; return ALTAR_TYPE; }
	YY_BREAK
case 87:
YY_RULE_SETUP
{ yylval.i=1; return UP_OR_DOWN; }
	YY_BREAK
case 88:
YY_RULE_SETUP
{ yylval.i=0; return UP_OR_DOWN; }
	YY_BREAK
case 89:
YY_RULE_SETUP
{ yylval.i=0; return BOOLEAN; }
	YY_BREAK
case 90:
YY_RULE_SETUP
{ yylval.i=1; return BOOLEAN; }
	YY_BREAK
case 91:
YY_RULE_SETUP
{ yylval.i=DUST; return ENGRAVING_TYPE; }
	YY_BREAK
case 92:
YY_RULE_SETUP
{ yylval.i=ENGRAVE; return ENGRAVING_TYPE; }
	YY_BREAK
case 93:
YY_RULE_SETUP
{ yylval.i=BURN; return ENGRAVING_TYPE; }
	YY_BREAK
case 94:
YY_RULE_SETUP
{ yylval.i=MARK; return ENGRAVING_TYPE; }
	YY_BREAK
case 95:
YY_RULE_SETUP
{ yylval.i=1; return CURSE_TYPE; }
	YY_BREAK
case 96:
YY_RULE_SETUP
{ yylval.i=2; return CURSE_TYPE; }
	YY_BREAK
case 97:
YY_RULE_SETUP
{ yylval.i=3; return CURSE_TYPE; }
	YY_BREAK
case 98:
YY_RULE_SETUP
{ return CONTAINED; }
	YY_BREAK
case 99:
YY_RULE_SETUP
{ yylval.i=NOTELEPORT; return FLAG_TYPE; }
	YY_BREAK
case 100:
YY_RULE_SETUP
{ yylval.i=HARDFLOOR; return FLAG_TYPE; }
	YY_BREAK
case 101:
YY_RULE_SETUP
{ yylval.i=NOMMAP; return FLAG_TYPE; }
	YY_BREAK
case 102:
YY_RULE_SETUP
{ yylval.i=ARBOREAL; return FLAG_TYPE; }	/* KMH */
	YY_BREAK
case 103:
YY_RULE_SETUP
{ yylval.i=SHORTSIGHTED; return FLAG_TYPE; }
	YY_BREAK
case 104:
YY_RULE_SETUP
{ yylval.i = atoi(yytext + 1); return PERCENT; }
	YY_BREAK
case 105:
YY_RULE_SETUP
{ yylval.i=atoi(yytext); return INTEGER; }
	YY_BREAK
case 106:
YY_RULE_SETUP
{ yytext[yyleng-1] = 0; /* Discard the trailing \" */
		  yylval.map = (char *) alloc(strlen(yytext+1)+1);
		  Strcpy(yylval.map, yytext+1); /* Discard the first \" */
		  return STRING; }
	YY_BREAK
case 107:
YY_RULE_SETUP
{ line_number++; }
	YY_BREAK
case 108:
YY_RULE_SETUP
;
	YY_BREAK
case 109:
YY_RULE_SETUP
{ yylval.i = yytext[2]; return CHAR; }
	YY_BREAK
case 110:
YY_RULE_SETUP
{ yylval.i = yytext[1]; return CHAR; }
	YY_BREAK
case 111:
YY_RULE_SETUP
{ return yytext[0]; }
	YY_BREAK
case 112:
YY_RULE_SETUP
ECHO;
	YY_BREAK
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(MAPC):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - yytext_ptr) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = yy_hold_char;

		if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between yy_current_buffer and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			yy_n_chars = yy_current_buffer->yy_n_chars;
			yy_current_buffer->yy_input_file = yyin;
			yy_current_buffer->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( yy_c_buf_p <= &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			yy_c_buf_p = yytext_ptr + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state();

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = yytext_ptr + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++yy_c_buf_p;
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = yy_c_buf_p;
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer() )
			{
			case EOB_ACT_END_OF_FILE:
				{
				yy_did_buffer_switch_on_eof = 0;

				if ( yywrap() )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					yy_c_buf_p = yytext_ptr + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				yy_c_buf_p =
					yytext_ptr + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				yy_c_buf_p =
				&yy_current_buffer->yy_ch_buf[yy_n_chars];

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of yylex */


/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */

static int yy_get_next_buffer()
	{
	register char *dest = yy_current_buffer->yy_ch_buf;
	register char *source = yytext_ptr;
	register int number_to_move, i;
	int ret_val;

	if ( yy_c_buf_p > &yy_current_buffer->yy_ch_buf[yy_n_chars + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( yy_current_buffer->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( yy_c_buf_p - yytext_ptr - YY_MORE_ADJ == 1 )
			{
			/* We matched a singled characater, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) (yy_c_buf_p - yytext_ptr) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		yy_n_chars = 0;

	else
		{
		int num_to_read =
			yy_current_buffer->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */
#ifdef YY_USES_REJECT
			YY_FATAL_ERROR(
"input buffer overflow, can't enlarge buffer because scanner uses REJECT" );
#else

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = yy_current_buffer;

			int yy_c_buf_p_offset =
				(int) (yy_c_buf_p - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int old_size = b->yy_buf_size + 2;
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					yy_flex_realloc2( (genericptr_t) b->yy_ch_buf,
							 b->yy_buf_size + 2, old_size );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = yy_current_buffer->yy_buf_size -
						number_to_move - 1;
#endif
			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&yy_current_buffer->yy_ch_buf[number_to_move]),
			yy_n_chars, num_to_read );
		}

	if ( yy_n_chars == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart( yyin );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			yy_current_buffer->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	yy_n_chars += number_to_move;
	yy_current_buffer->yy_ch_buf[yy_n_chars] = YY_END_OF_BUFFER_CHAR;
	yy_current_buffer->yy_ch_buf[yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	yytext_ptr = &yy_current_buffer->yy_ch_buf[0];

	return ret_val;
	}


/* yy_get_previous_state - get the state just before the EOB char was reached */

static yy_state_type yy_get_previous_state()
	{
	register yy_state_type yy_current_state;
	register char *yy_cp;

	yy_current_state = yy_start;
	yy_current_state += YY_AT_BOL();

	for ( yy_cp = yytext_ptr + YY_MORE_ADJ; yy_cp < yy_c_buf_p; ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			yy_last_accepting_state = yy_current_state;
			yy_last_accepting_cpos = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 640 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
	}


/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */

static yy_state_type yy_try_NUL_trans( yy_current_state )
yy_state_type yy_current_state;
	{
	register int yy_is_jam;
	register char *yy_cp = yy_c_buf_p;

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		yy_last_accepting_state = yy_current_state;
		yy_last_accepting_cpos = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 640 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 639);

	return yy_is_jam ? 0 : yy_current_state;
	}


#ifndef YY_NO_UNPUT
static void yyunput( c, yy_bp )
int c;
register char *yy_bp;
	{
	register char *yy_cp = yy_c_buf_p;

	/* undo effects of setting up yytext */
	*yy_cp = yy_hold_char;

	if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = yy_n_chars + 2;
		register char *dest = &yy_current_buffer->yy_ch_buf[
					yy_current_buffer->yy_buf_size + 2];
		register char *source =
				&yy_current_buffer->yy_ch_buf[number_to_move];

		while ( source > yy_current_buffer->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		yy_n_chars = yy_current_buffer->yy_buf_size;

		if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;


	yytext_ptr = yy_bp;
	yy_hold_char = *yy_cp;
	yy_c_buf_p = yy_cp;
	}
#endif	/* ifndef YY_NO_UNPUT */


static int input()
	{
	int c;

	*yy_c_buf_p = yy_hold_char;

	if ( *yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( yy_c_buf_p < &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			/* This was really a NUL. */
			*yy_c_buf_p = '\0';

		else
			{ /* need more input */
			yytext_ptr = yy_c_buf_p;
			++yy_c_buf_p;

			switch ( yy_get_next_buffer() )
				{
				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap() )
						{
						yy_c_buf_p =
						yytext_ptr + YY_MORE_ADJ;
						return EOF;
						}

					if ( ! yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
					return input();
					}

				case EOB_ACT_CONTINUE_SCAN:
					yy_c_buf_p = yytext_ptr + YY_MORE_ADJ;
					break;

				case EOB_ACT_LAST_MATCH:
					YY_FATAL_ERROR(
					"unexpected last match in input()" );
				}
			}
		}

	c = *(unsigned char *) yy_c_buf_p;	/* cast for 8-bit char's */
	*yy_c_buf_p = '\0';	/* preserve yytext */
	yy_hold_char = *++yy_c_buf_p;

	yy_current_buffer->yy_at_bol = (c == '\n');

	return c;
	}


void yyrestart( input_file )
FILE *input_file;
	{
	if ( ! yy_current_buffer )
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE );

	yy_init_buffer( yy_current_buffer, input_file );
	yy_load_buffer_state();
	}


void yy_switch_to_buffer( new_buffer )
YY_BUFFER_STATE new_buffer;
	{
	if ( yy_current_buffer == new_buffer )
		return;

	if ( yy_current_buffer )
		{
		/* Flush out information for old buffer. */
		*yy_c_buf_p = yy_hold_char;
		yy_current_buffer->yy_buf_pos = yy_c_buf_p;
		yy_current_buffer->yy_n_chars = yy_n_chars;
		}

	yy_current_buffer = new_buffer;
	yy_load_buffer_state();

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	yy_did_buffer_switch_on_eof = 1;
	}


void yy_load_buffer_state()
	{
	yy_n_chars = yy_current_buffer->yy_n_chars;
	yytext_ptr = yy_c_buf_p = yy_current_buffer->yy_buf_pos;
	yyin = yy_current_buffer->yy_input_file;
	yy_hold_char = *yy_c_buf_p;
	}


YY_BUFFER_STATE yy_create_buffer( file, size )
FILE *file;
int size;
	{
	YY_BUFFER_STATE b;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yy_flex_alloc( b->yy_buf_size + 2 );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer( b, file );

	return b;
	}


void yy_delete_buffer( b )
YY_BUFFER_STATE b;
	{
	if ( ! b )
		return;

	if ( b == yy_current_buffer )
		yy_current_buffer = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yy_flex_free( (genericptr_t) b->yy_ch_buf );

	yy_flex_free( (genericptr_t) b );
	}


#ifndef YY_ALWAYS_INTERACTIVE
#ifndef YY_NEVER_INTERACTIVE
extern int FDECL(isatty, (int));
#endif
#endif

void yy_init_buffer( b, file )
YY_BUFFER_STATE b;
FILE *file;
	{
	yy_flush_buffer( b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

#ifdef YY_ALWAYS_INTERACTIVE
	b->yy_is_interactive = 1;
#else
#ifdef YY_NEVER_INTERACTIVE
	b->yy_is_interactive = 0;
#else
	b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
#endif
#endif
	}


void yy_flush_buffer( b )
YY_BUFFER_STATE b;
	{
	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == yy_current_buffer )
		yy_load_buffer_state();
	}



#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yy_fatal_error( msg )
const char msg[];
	{
	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
	}



/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		yytext[yyleng] = yy_hold_char; \
		yy_c_buf_p = yytext + n - YY_MORE_ADJ; \
		yy_hold_char = *yy_c_buf_p; \
		*yy_c_buf_p = '\0'; \
		yyleng = n; \
		} \
	while ( 0 )


/* Internal utility routines. */

#ifndef yytext_ptr
static void yy_flex_strncpy( s1, s2, n )
char *s1;
const char *s2;
int n;
	{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
	}
#endif


static genericptr_t yy_flex_alloc( size )
yy_size_t size;
	{
	return (genericptr_t) alloc((unsigned)size);
	}

/* we want to avoid use of realloc(), so we require that caller supply the
   size of the old block of memory */
static genericptr_t yy_flex_realloc2( ptr, size, old_size )
genericptr_t ptr;
yy_size_t size;
int old_size;
	{
	genericptr_t outptr = yy_flex_alloc(size);

	if (ptr) {
	    char *p = (char *) outptr, *q = (char *) ptr;

	    while (--old_size >= 0) *p++ = *q++;
	    yy_flex_free(ptr);
	}
	return outptr;
	}

static void yy_flex_free( ptr )
genericptr_t ptr;
	{
	free( ptr );
	}

/*flexhack.skl*/

#ifdef	AMIGA
long *alloc(n)
	unsigned n;
{
	return ((long *)malloc (n));
}
#endif

/* routine to switch to another input file; needed for flex */
void init_yyin( input_f )
FILE *input_f;
{
#if defined(FLEX_SCANNER) || defined(FLEXHACK_SCANNER)
	if (yyin)
	    yyrestart(input_f);
	else
#endif
	    yyin = input_f;
}
/* analogous routine (for completeness) */
void init_yyout( output_f )
FILE *output_f;
{
	yyout = output_f;
}

/*lev_comp.l*/
